# M106 Winter 2021, Data-driven UQ
# by Yoonsang Lee (yoonsang.lee@dartmouth.edu)
#
# Lecture 9: MCMC
#
# Markov Chain Monte Carlo sample using
# the Metropolis-Hastings Algorithm
#
# Last update: Jan 25, 2020

import numpy as np
from matplotlib import pyplot as plt

def target(x):
    #return 1/(1+np.power(x,2))/np.pi
    return np.exp(-(x-5)**2)+np.exp(-(x+5)**2)
b=10; # variance of the proposal Gaussian density

niters = 50000
naccept = 0

theta = 0 # initial value for the Markov chain

samples = np.zeros(niters+1)
samples[0] = theta
for i in range(niters):
    theta_p = theta + np.random.randn()*b;
    rho = min(1, target(theta_p)/target(theta ))
    u = np.random.uniform()
    if u < rho:
        naccept += 1
        theta = theta_p
    samples[i+1] = theta

print("acceptance rate:", naccept/niters)
plt.clf();
plt.hist(samples,50,density=True)
